import re
import json
import jsonlines
from pathlib import Path

path_index = Path("index.txt")
curr_index = int(path_index.read_text().strip()) if path_index.exists() else 0
index = 0
oclc = set()
isbn = set()


def newitem(item1):
    item2 = {"temas": [], "isbns": [], "descripcion": []}
    keysmap = {
        "title": "titulo",
        "subjectsText": "temas",
        "genre": "temas",
        "subjects": "temas",
        "series": "serie",
        "physicalDescription": "paginas",
        "oclcNumber": "oclc",
        "isbns": "isbns",
        "isbn13": "isbn13",
        "publicationDate": "año",
        "publisher": "editorial",
        "summary": "descripcion",
        "abstract": "descripcion",
        "summaries": "descripcion",
        "creator": "autor",
    }
    if "book" not in item1["generalFormat"].lower():
        return None
    for key1 in item1.keys():
        if not item1[key1] or key1 not in keysmap.keys():
            continue
        key2 = keysmap[key1]
        if key2 not in item2.keys():
            item2[key2] = toint(item1[key1])
        else:
            if isinstance(item1[key1], list):
                [item2[key2].append(toint(val)) for val in item1[key1]]
            else:
                item2[key2].append(toint(item1[key1]))
    [item2.setdefault(v, "") for v in keysmap.values()]
    return item2


def additem(dump, index, prefix):
    filename = "%s_books_annas_archive_es.jsonl" % prefix
    if index > curr_index:
        print(index, dump[0:99])
        with open(filename, "a") as myfile:
            myfile.write(dump)
    else:
        print("Ignoring item %s" % index)


def toint(val):
    return int(val) if val.isdigit() else val


def setint(val1, rgx):
    val2 = re.findall(rgx, str(val1))
    return int(val2[0]) if val2 else None


with jsonlines.open("annas_archive_es.jsonl") as reader:
    for obj in reader:
        try:
            index += 1
            item = newitem(obj["metadata"]["record"])
            if not item:
                continue
            item["año"] = setint(item["año"], r"(\d+?)")
            item["paginas"] = setint(item["paginas"], r"(\d+?)\s*pages")
            if item["isbns"]:
                isbns = list(map(lambda e: str(e), item["isbns"]))
                extra = "ISBN Relacionados: %s" % ", ".join(isbns)
                item["descripcion"].append(extra)
            item["descripcion"] = "\n\n".join(item["descripcion"])
            if item["oclc"] not in oclc:
                oclc.add(item["oclc"])
                dump = json.dumps(item, ensure_ascii=False).encode("utf8")
                dump = dump.decode() + "\n"
                if not item["isbn13"]:
                    additem(dump, index, "unregistered")
                elif item["isbn13"] and item["isbn13"] not in isbn:
                    isbn.add(item["isbn13"])
                    additem(dump, index, "registered")
        except Exception as err:
            strs = (type(err).__name__, err.__traceback__.tb_lineno, str(err))
            print("%s at line %s: %s" % strs)
            pass
